<?php
namespace FiloBlu\BirthdayCoupon\Model\Magento\Quote\Quote;

use Magento\Quote\Model\Quote\Address\Total\Collector;
use Magento\Quote\Model\Quote\Address\Total\CollectorFactory;

class TotalsCollector extends \Magento\Quote\Model\Quote\TotalsCollector
{
    protected $_birthdayHelper;
    protected $totalCollector;
    protected $totalCollectorFactory;
    protected $eventManager;
    protected $storeManager;
    protected $totalFactory;
    protected $collectorList;
    protected $quoteValidator;
    protected $shippingFactory;
    protected $shippingAssignmentFactory;

    public function __construct(
        \FiloBlu\BirthdayCoupon\Helper\Data $birthdayHelper,
        Collector $totalCollector,
        CollectorFactory $totalCollectorFactory,
        \Magento\Framework\Event\ManagerInterface $eventManager,
        \Magento\Store\Model\StoreManagerInterface $storeManager,
        \Magento\Quote\Model\Quote\Address\TotalFactory $totalFactory,
        \Magento\Quote\Model\Quote\TotalsCollectorList $collectorList,
        \Magento\Quote\Model\ShippingFactory $shippingFactory,
        \Magento\Quote\Model\ShippingAssignmentFactory $shippingAssignmentFactory,
        \Magento\Quote\Model\QuoteValidator $quoteValidator
    ) {
        $this->_birthdayHelper = $birthdayHelper;
        $this->totalCollector = $totalCollector;
        $this->totalCollectorFactory = $totalCollectorFactory;
        $this->eventManager = $eventManager;
        $this->storeManager = $storeManager;
        $this->totalFactory = $totalFactory;
        $this->collectorList = $collectorList;
        $this->shippingFactory = $shippingFactory;
        $this->shippingAssignmentFactory = $shippingAssignmentFactory;
        $this->quoteValidator = $quoteValidator;
        parent::__construct($totalCollector, $totalCollectorFactory, $eventManager, $storeManager, $totalFactory, $collectorList, $shippingFactory, $shippingAssignmentFactory, $quoteValidator);
    }

    protected function _validateCouponCode(\Magento\Quote\Model\Quote $quote)
    {
        $code = $quote->getData('coupon_code');
        $addressHasCoupon = false;
        if (strlen($code)) {
            if($this->_birthdayHelper->isModuleEnabled() && $this->_birthdayHelper->isBirthdayCoupon($code) &&
                ($quote->getData('customer_email') != '' || $quote->getData('customer_id') != '')) {
                    $validate = $this->_birthdayHelper->validateBirthdayCoupon($quote);
                    if($validate->result == 'true') {
                        $addressHasCoupon = true;
                    }
            }
            else {
                $addresses = $quote->getAllAddresses();
                if (count($addresses) > 0) {
                    foreach ($addresses as $address) {
                        if ($address->hasCouponCode()) {
                            $addressHasCoupon = true;
                        }
                    }
                }
            }

            if (!$addressHasCoupon) {
                $quote->setCouponCode('');
            }
        }
        return $this;
    }
}